@@ -26,6 +26,15 @@ package Bugzilla;
 
 use strict;
 
+# We want any compile errors to get to the browser, if possible.
+BEGIN {
+    # This makes sure we're in a CGI.
+    if ($ENV{SERVER_SOFTWARE} && !$ENV{MOD_PERL}) {
+        require CGI::Carp;
+        CGI::Carp->import('fatalsToBrowser');
+    }
+}
+
 use Bugzilla::Config;
 use Bugzilla::Constants;
 use Bugzilla::Auth;
@@ -38,8 +47,10 @@ use Bugzilla::User;
 use Bugzilla::Error;
 use Bugzilla::Util;
 use Bugzilla::Field;
+use Bugzilla::Flag;
 
 use File::Basename;
+use File::Spec::Functions;
 use Safe;
 
 # This creates the request cache for non-mod_perl installations.
@@ -65,20 +76,11 @@ use constant SHUTDOWNHTML_EXIT_SILENTLY => [
 # Global Code
 #####################################################################
 
-# The following subroutine is for debugging purposes only.
-# Uncommenting this sub and the $::SIG{__DIE__} trap underneath it will
-# cause any fatal errors to result in a call stack trace to help track
-# down weird errors.
-#
-#sub die_with_dignity {
-#    use Carp ();
-#    my ($err_msg) = @_;
-#    print $err_msg;
-#    Carp::confess($err_msg);
-#}
-#$::SIG{__DIE__} = \&Bugzilla::die_with_dignity;
+# $::SIG{__DIE__} = i_am_cgi() ? \&CGI::Carp::confess : \&Carp::confess;
 
+# Note that this is a raw subroutine, not a method, so $class isn't available.
 sub init_page {
+    (binmode STDOUT, ':utf8') if Bugzilla->params->{'utf8'};
 
     # Some environment variables are not taint safe
     delete @::ENV{'PATH', 'IFS', 'CDPATH', 'ENV', 'BASH_ENV'};
@@ -125,7 +127,13 @@ sub init_page {
 
         # For security reasons, log out users when Bugzilla is down.
         # Bugzilla->login() is required to catch the logincookie, if any.
-        my $user = Bugzilla->login(LOGIN_OPTIONAL);
+        my $user;
+        eval { $user = Bugzilla->login(LOGIN_OPTIONAL); };
+        if ($@) {
+            # The DB is not accessible. Use the default user object.
+            $user = Bugzilla->user;
+            $user->{settings} = {};
+        }
         my $userid = $user->id;
         Bugzilla->logout();
 
@@ -160,40 +168,42 @@ init_page() if !$ENV{MOD_PERL};
 
 sub template {
     my $class = shift;
-    request_cache()->{language} = "";
-    request_cache()->{template} ||= Bugzilla::Template->create();
-    return request_cache()->{template};
+    $class->request_cache->{language} = "";
+    $class->request_cache->{template} ||= Bugzilla::Template->create();
+    return $class->request_cache->{template};
 }
 
 sub template_inner {
     my ($class, $lang) = @_;
-    $lang = defined($lang) ? $lang : (request_cache()->{language} || "");
-    request_cache()->{language} = $lang;
-    request_cache()->{"template_inner_$lang"} ||= Bugzilla::Template->create();
-    return request_cache()->{"template_inner_$lang"};
+    $lang = defined($lang) ? $lang : ($class->request_cache->{language} || "");
+    $class->request_cache->{language} = $lang;
+    $class->request_cache->{"template_inner_$lang"}
+        ||= Bugzilla::Template->create();
+    return $class->request_cache->{"template_inner_$lang"};
 }
 
 sub cgi {
     my $class = shift;
-    request_cache()->{cgi} ||= new Bugzilla::CGI();
-    return request_cache()->{cgi};
+    $class->request_cache->{cgi} ||= new Bugzilla::CGI();
+    return $class->request_cache->{cgi};
 }
 
 sub localconfig {
-    request_cache()->{localconfig} ||= read_localconfig();
-    return request_cache()->{localconfig};
+    my $class = shift;
+    $class->request_cache->{localconfig} ||= read_localconfig();
+    return $class->request_cache->{localconfig};
 }
 
 sub params {
     my $class = shift;
-    request_cache()->{params} ||= Bugzilla::Config::read_param_file();
-    return request_cache()->{params};
+    $class->request_cache->{params} ||= Bugzilla::Config::read_param_file();
+    return $class->request_cache->{params};
 }
 
 sub user {
     my $class = shift;
-    request_cache()->{user} ||= new Bugzilla::User;
-    return request_cache()->{user};
+    $class->request_cache->{user} ||= new Bugzilla::User;
+    return $class->request_cache->{user};
 }
 
 sub set_user {
@@ -203,31 +213,25 @@ sub set_user {
 
 sub sudoer {
     my $class = shift;    
-    return request_cache()->{sudoer};
+    return $class->request_cache->{sudoer};
 }
 
 sub sudo_request {
-    my $class = shift;
-    my $new_user = shift;
-    my $new_sudoer = shift;
-
-    request_cache()->{user}   = $new_user;
-    request_cache()->{sudoer} = $new_sudoer;
-
+    my ($class, $new_user, $new_sudoer) = @_;
+    $class->request_cache->{user}   = $new_user;
+    $class->request_cache->{sudoer} = $new_sudoer;
     # NOTE: If you want to log the start of an sudo session, do it here.
-
-    return;
 }
 
 sub login {
     my ($class, $type) = @_;
 
-    return Bugzilla->user if Bugzilla->user->id;
+    return $class->user if $class->user->id;
 
     my $authorizer = new Bugzilla::Auth();
-    $type = LOGIN_REQUIRED if Bugzilla->cgi->param('GoAheadAndLogIn');
+    $type = LOGIN_REQUIRED if $class->cgi->param('GoAheadAndLogIn');
     if (!defined $type || $type == LOGIN_NORMAL) {
-        $type = Bugzilla->params->{'requirelogin'} ? LOGIN_REQUIRED : LOGIN_NORMAL;
+        $type = $class->params->{'requirelogin'} ? LOGIN_REQUIRED : LOGIN_NORMAL;
     }
     my $authenticated_user = $authorizer->login($type);
     
@@ -250,8 +254,8 @@ sub login {
         !($sudo_target->in_group('bz_sudo_protect'))
        )
     {
-        Bugzilla->set_user($sudo_target);
-        request_cache()->{sudoer} = $authenticated_user;
+        $class->set_user($sudo_target);
+        $class->request_cache->{sudoer} = $authenticated_user;
         # And make sure that both users have the same Auth object,
         # since we never call Auth::login for the sudo target.
         $sudo_target->set_authorizer($authenticated_user->authorizer);
@@ -259,21 +263,29 @@ sub login {
         # NOTE: If you want to do any special logging, do it here.
     }
     else {
-        Bugzilla->set_user($authenticated_user);
+        $class->set_user($authenticated_user);
+    }
+
+    # We run after the login has completed since
+    # some of the checks in ssl_require_redirect
+    # look for Bugzilla->user->id to determine 
+    # if redirection is required.
+    if (i_am_cgi() && ssl_require_redirect()) {
+        $class->cgi->require_https($class->params->{'sslbase'});
     }
     
-    return Bugzilla->user;
+    return $class->user;
 }
 
 sub logout {
     my ($class, $option) = @_;
 
     # If we're not logged in, go away
-    return unless user->id;
+    return unless $class->user->id;
 
     $option = LOGOUT_CURRENT unless defined $option;
     Bugzilla::Auth::Persist::Cookie->logout({type => $option});
-    Bugzilla->logout_request() unless $option eq LOGOUT_KEEP_CURRENT;
+    $class->logout_request() unless $option eq LOGOUT_KEEP_CURRENT;
 }
 
 sub logout_user {
@@ -292,35 +304,53 @@ sub logout_user_by_id {
 
 # hack that invalidates credentials for a single request
 sub logout_request {
-    delete request_cache()->{user};
-    delete request_cache()->{sudoer};
+    my $class = shift;
+    delete $class->request_cache->{user};
+    delete $class->request_cache->{sudoer};
     # We can't delete from $cgi->cookie, so logincookie data will remain
     # there. Don't rely on it: use Bugzilla->user->login instead!
 }
 
 sub dbh {
     my $class = shift;
-
     # If we're not connected, then we must want the main db
-    request_cache()->{dbh} ||= request_cache()->{dbh_main} 
+    $class->request_cache->{dbh} ||= $class->request_cache->{dbh_main} 
         = Bugzilla::DB::connect_main();
 
-    return request_cache()->{dbh};
+    return $class->request_cache->{dbh};
+}
+
+sub languages {
+    my $class = shift;
+    return $class->request_cache->{languages}
+        if $class->request_cache->{languages};
+
+    my @files = glob(catdir(bz_locations->{'templatedir'}, '*'));
+    my @languages;
+    foreach my $dir_entry (@files) {
+        # It's a language directory only if it contains "default" or
+        # "custom". This auto-excludes CVS directories as well.
+        next unless (-d catdir($dir_entry, 'default')
+                  || -d catdir($dir_entry, 'custom'));
+        $dir_entry = basename($dir_entry);
+        # Check for language tag format conforming to RFC 1766.
+        next unless $dir_entry =~ /^[a-zA-Z]{1,8}(-[a-zA-Z]{1,8})?$/;
+        push(@languages, $dir_entry);
+    }
+    return $class->request_cache->{languages} = \@languages;
 }
 
 sub error_mode {
-    my $class = shift;
-    my $newval = shift;
+    my ($class, $newval) = @_;
     if (defined $newval) {
-        request_cache()->{error_mode} = $newval;
+        $class->request_cache->{error_mode} = $newval;
     }
-    return request_cache()->{error_mode}
+    return $class->request_cache->{error_mode}
         || Bugzilla::Constants::ERROR_MODE_WEBPAGE;
 }
 
 sub usage_mode {
-    my $class = shift;
-    my $newval = shift;
+    my ($class, $newval) = @_;
     if (defined $newval) {
         if ($newval == USAGE_MODE_BROWSER) {
             $class->error_mode(ERROR_MODE_WEBPAGE);
@@ -338,9 +368,9 @@ sub usage_mode {
             ThrowCodeError('usage_mode_invalid',
                            {'invalid_usage_mode', $newval});
         }
-        request_cache()->{usage_mode} = $newval;
+        $class->request_cache->{usage_mode} = $newval;
     }
-    return request_cache()->{usage_mode}
+    return $class->request_cache->{usage_mode}
         || Bugzilla::Constants::USAGE_MODE_BROWSER;
 }
 
@@ -369,15 +399,15 @@ sub installation_answers {
 sub switch_to_shadow_db {
     my $class = shift;
 
-    if (!request_cache()->{dbh_shadow}) {
-        if (Bugzilla->params->{'shadowdb'}) {
-            request_cache()->{dbh_shadow} = Bugzilla::DB::connect_shadow();
+    if (!$class->request_cache->{dbh_shadow}) {
+        if ($class->params->{'shadowdb'}) {
+            $class->request_cache->{dbh_shadow} = Bugzilla::DB::connect_shadow();
         } else {
-            request_cache()->{dbh_shadow} = request_cache()->{dbh_main};
+            $class->request_cache->{dbh_shadow} = request_cache()->{dbh_main};
         }
     }
 
-    request_cache()->{dbh} = request_cache()->{dbh_shadow};
+    $class->request_cache->{dbh} = $class->request_cache->{dbh_shadow};
     # we have to return $class->dbh instead of {dbh} as
     # {dbh_shadow} may be undefined if no shadow DB is used
     # and no connection to the main DB has been established yet.
@@ -387,7 +417,7 @@ sub switch_to_shadow_db {
 sub switch_to_main_db {
     my $class = shift;
 
-    request_cache()->{dbh} = request_cache()->{dbh_main};
+    $class->request_cache->{dbh} = $class->request_cache->{dbh_main};
     # We have to return $class->dbh instead of {dbh} as
     # {dbh_main} may be undefined if no connection to the main DB
     # has been established yet.
@@ -404,10 +434,22 @@ sub get_fields {
     return @$fields;
 }
 
-sub custom_field_names {
-    # Get a list of custom fields and convert it into a list of their names.
-    return map($_->{name}, 
-               @{Bugzilla::Field->match({ custom=>1, obsolete=>0 })});
+sub active_custom_fields {
+    my $class = shift;
+    if (!exists $class->request_cache->{active_custom_fields}) {
+        $class->request_cache->{active_custom_fields} =
+          Bugzilla::Field->match({ custom => 1, obsolete => 0 });
+    }
+    return @{$class->request_cache->{active_custom_fields}};
+}
+
+sub has_flags {
+    my $class = shift;
+
+    if (!defined $class->request_cache->{has_flags}) {
+        $class->request_cache->{has_flags} = Bugzilla::Flag::has_flags();
+    }
+    return $class->request_cache->{has_flags};
 }
 
 sub hook_args {
@@ -426,14 +468,16 @@ sub request_cache {
 
 # Private methods
 
-# Per process cleanup
+# Per-process cleanup. Note that this is a plain subroutine, not a method,
+# so we don't have $class available.
 sub _cleanup {
-
-    # When we support transactions, need to ->rollback here
-    my $main   = request_cache()->{dbh_main};
-    my $shadow = request_cache()->{dbh_shadow};
-    $main->disconnect if $main;
-    $shadow->disconnect if $shadow && Bugzilla->params->{"shadowdb"};
+    my $main   = Bugzilla->request_cache->{dbh_main};
+    my $shadow = Bugzilla->request_cache->{dbh_shadow};
+    foreach my $dbh ($main, $shadow) {
+        next if !$dbh;
+        $dbh->bz_rollback_transaction() if $dbh->bz_in_transaction;
+        $dbh->disconnect;
+    }
     undef $_request_cache;
 }
 
@@ -626,6 +670,11 @@ used to automatically answer or skip prompts.
 
 The current database handle. See L<DBI>.
 
+=item C<languages>
+
+Currently installed languages.
+Returns a reference to a list of RFC 1766 language tags of installed languages.
+
 =item C<switch_to_shadow_db>
 
 Switch from using the main database to using the shadow database.
