@@ -62,7 +62,9 @@ sub REQUIRED_MODULES {
         # Perl 5.10 requires CGI 3.33 due to a taint issue when
         # uploading attachments, see bug 416382.
         # Require CGI 3.21 for -httponly support, see bug 368502.
-        version => (vers_cmp($perl_ver, '5.10') > -1) ? '3.33' : '3.21'
+        version => (vers_cmp($perl_ver, '5.10') > -1) ? '3.33' : '3.21',
+        # CGI::Carp in 3.46 and 3.47 breaks Template Toolkit
+        blacklist => ['^3\.46$', '^3\.47$'],
     },
     {
         package => 'TimeDate',
@@ -79,15 +81,18 @@ sub REQUIRED_MODULES {
         module  => 'DBI',
         version => '1.41'
     },
+    # 2.22 fixes various problems related to UTF8 strings in hash keys,
+    # as well as line endings on Windows.
     {
         package => 'Template-Toolkit',
         module  => 'Template',
-        version => '2.15'
+        version => ON_WINDOWS ? '2.22' : '2.15'
     },
     {
         package => 'Email-Send',
         module  => 'Email::Send',
-        version => ON_WINDOWS ? '2.16' : '2.00'
+        version => ON_WINDOWS ? '2.16' : '2.00',
+        blacklist => ['^2\.196$']
     },
     {
         package => 'Email-MIME',
