@@ -351,8 +351,7 @@
 'bug/time.html.tmpl' => [
   'time_unit FILTER format(\'%.1f\')',
   'time_unit FILTER format(\'%.2f\')',
-  '(act / (act + rem)) * 100
-       FILTER format("%d")',
+  "(act / (act + rem)) * 100 \n       FILTER format(\"%d\")",
 ],
 
 'bug/votes/list-for-bug.html.tmpl' => [
@@ -382,7 +381,6 @@
 ],
 
 'bug/create/create-guided.html.tmpl' => [
-  'matches.0',
   'tablecolour',
   'sel',
   'productstring',
