@@ -24,10 +24,9 @@ use warnings;
 
 # MTAs may call this script from any directory, but it should always
 # run from this one so that it can find its modules.
-BEGIN {
-    require File::Basename;
-    chdir(File::Basename::dirname($0)); 
-}
+use Cwd qw(abs_path);
+use File::Basename qw(dirname);
+BEGIN { chdir dirname(abs_path($0)); }
 
 use lib qw(. lib);
 
@@ -47,6 +46,7 @@ use Bugzilla::Error;
 use Bugzilla::Mailer;
 use Bugzilla::User;
 use Bugzilla::Util;
+use Bugzilla::Token;
 
 #############
 # Constants #
@@ -87,7 +87,7 @@ sub parse_mail {
     debug_print("Body:\n" . $body, 3);
 
     $body = remove_leading_blank_lines($body);
-    my @body_lines = split("\n", $body);
+    my @body_lines = split(/\r?\n/s, $body);
 
     # If there are fields specified.
     if ($body =~ /^\s*@/s) {
@@ -202,6 +202,7 @@ sub process_bug {
         $cgi->param(-name => $field, -value => $fields{$field});
     }
     $cgi->param('longdesclength', scalar $bug->longdescs);
+    $cgi->param('token', issue_hash_token([$bug->id, $bug->delta_ts]));
 
     require 'process_bug.cgi';
 }
